'''
Fourier-Transformation

Zeit im Testlauf mit 64x64 Pixel ca. 7 Sekunden.

'''

import matplotlib.pyplot as plt
import matplotlib.colors as colors
import numpy as np
import math
import time




def fourierTransformFunction1DRows(d, image, x, y):
    
    h, w = image.shape[0:2]
    resultRe = 0
    resultIm = 0
    
    for i in xrange(0, w):
        value = image[y, i]
        cosValue = math.cos(d * 2 * math.pi * x * i / w)
        sinValue = math.sin(d * 2 * math.pi * x * i / w)
        resultRe = resultRe + value[0] * cosValue - value[1] * sinValue
        resultIm = resultIm + value[0] * sinValue + value[1] * cosValue
            
    return resultRe, resultIm



def fourierTransformFunction1DCols(d, image, x, y):
    
    h, w = image.shape[0:2]
    resultRe = 0
    resultIm = 0
    
    for i in xrange(0, h):
        value = image[i, x]
        cosValue = math.cos(d * 2 * math.pi * y * i / h)
        sinValue = math.sin(d * 2 * math.pi * y * i / h)
        resultRe = resultRe + value[0] * cosValue - value[1] * sinValue
        resultIm = resultIm + value[0] * sinValue + value[1] * cosValue
            
    return resultRe, resultIm



def fourierTransform(data, h, w, d):
        
    result_tmp = np.zeros((h, w, 2))   
    for y in xrange(h):
        print "Dim 1: Working on row", y, "..."
        for x in xrange(w):
            result_tmp[y, x] = fourierTransformFunction1DCols(d, data, x, y)
       
    result = np.zeros((h, w, 2))     
    for y in xrange(h):
        print "Dim 2: Working on row", y, "..."
        for x in xrange(w):
            result[y, x] = fourierTransformFunction1DRows(d, result_tmp, x, y)
    
    if d == 1:
        result = result / (w * h)
            
    return result


def fourierTransformForward(image):

    h, w = image.shape[0:2]      
    data = np.array([[ (v, 0) for v in row] for row in image])      
    return fourierTransform(data, h, w, -1)


def fourierTransformBackward(data):

    h, w = data.shape[0:2]       
    return fourierTransform(data, h, w, 1)
    
 
 
def shift(data):
    result = np.zeros_like(data)
    result_tmp = np.zeros_like(data)
    h, w = data.shape
    result_tmp[0:h / 2, :] = data[h / 2:h, :]
    result_tmp[h / 2:h, :] = data[0:h / 2, :]
    
    result[:, 0:w / 2] = result_tmp[:, w / 2:w]
    result[:, w / 2:w] = result_tmp[:, 0:w / 2]
    
    return result    


def aufgabe1():
    input = plt.imread('nerds_64.png')
    
    start = time.time()
    resultForward = fourierTransformForward(input)
    end = time.time()
    resultBackward = fourierTransformBackward(resultForward)
    
    print "Time taken to compute FT:", end - start
    
    resultForwardAmp = np.array([[math.sqrt(x*x+y*y) for x, y in row] for row in resultForward])
    resultForwardAmp = shift(np.log(1 + resultForwardAmp))   
    resultBackwardRe = resultBackward[:, :, 0]
    
    plt.figure(1)
    plt.subplot(131)
    plt.title("Original")
    plt.imshow(input, cmap=plt.cm.gray, norm=colors.NoNorm())
    plt.subplot(132)
    plt.title("Forward FT")
    plt.imshow(resultForwardAmp, cmap=plt.cm.gray)
    plt.subplot(133)
    plt.title("Backward FT")
    plt.imshow(resultBackwardRe, cmap=plt.cm.gray, norm=colors.NoNorm())
    plt.show()
    


aufgabe1()
